# AM2CM

- Tool for translating Ambari configurations (Ambari Blueprint) to Cloudera Manager configurations (CM Deployment Template)

How to build the code:
----------------------
- Download the source using "git clone" and then run below command to build WAR file:
mvn clean install
- Generates target/am2cm folder with jar and configs.
- [Running a SonarQube scan](docs/sonar.md)


How does it work:
-------------------
 - Reads the configurations and hosts from Ambari Blueprints
 - Translates configurations using cm-config-mapping.ini and cm-config-mapping-custom.ini mappings.
 - Groups the Service configs using some of the configs defined in service-config.ini
 - Stop services from Ambari
 - Generates CM deployment template
 - Activate and Start the services from Cloudera Manager.



How to add new config mappings
------------------------------
1. If you find any of the configs are missing then add entry in cm-config-mapping-custom.ini under that Service/Component block
   Ex:
   Role param specs for MASTER:
   hbase_master_java_heapsize=hbase_master_heapsize


How to add new services
------------------------------
1. Add Service-Component mappings under CONFIGS-SERVICE-COMPONENT-MAPPINGS in service-config.ini
Ex: HBASE=HBASE_MASTER,HBASE_REGIONSERVER

2. Add Service-Config type mappings under CONFIGS-SERVICE-CONFIGTYPE-MAPPINGS in service-config.ini
Ex: HBASE=hbase-atlas-application-properties,hbase-env

3. Add Component/Role mappings under CONFIGS-COMPONENT-MAPPINGS if component names are different between Ambari - CM
Ex: SECONDARY_NAMENODE=SECONDARYNAMENODE

4. Add Service dependency mappings under CONFIGS-SERVICE-DEPENDENCY-MAPPINGS
Ex: HBASE=zookeeper_service,hdfs_service

5. Add entries under CONFIGS-SERVICE-CONFIG-IGNORE-MAPPINGS to ignore any of configs for that services.

6. If any of the configs needs translations then have to add new Java class to do the conversion by extending Service.java


How to use AM2CM tool?
----------------------

1. Copy Ambari blueprint to am2cm/conf/blueprint.json file, OR via command line specify where the ambari blueprint is present as following:
```
   # /PATH/am2cm.sh  --blueprint-file PATH/TO/Amb_blueprint.json  --deployment-template-file /PATH/TO/cm_deployment_template.json --source-hdp-version hdp3
   (OR)
   # /PATH/am2cm.sh  -bp PATH/TO/Amb_blueprint.json  -dt /PATH/TO/cm_deployment_template.json -hdp hdp3
   # /PATH/am2cm.sh  --blueprint-file PATH/TO/Amb_blueprint.json  --deployment-template-file /PATH/TO/cm_deployment_template.json --source-hdp-version hdp3
   (OR)
   # /PATH/am2cm.sh  -bp PATH/TO/Amb_blueprint.json  -dt /PATH/TO/cm_deployment_template.json -hdp hdp3
```
If you would like to skip any pre-upgrade validations of your blueprint, please supply the following option as well:
```
  -spu
  (OR)
  --skip-pre-upgrade
```
If you would like to run ONLY the pre-upgrade validations (without generating any cluster deployment template output), please supply the following option:
```
  -opu
  (OR)
  --only-pre-upgrade
```
Note: Both skip-pre-upgrade and only-pre-upgrade are set to **false** by default. Using both options is permitted, however, *skip-pre-upgrade* takes precedence.

Note: hdp source version can be "hdp2" or "hdp3"

2. Update Parcels information in user-settings.ini file if it is different from defaults.

3. set JAVA_HOME 

4.  Now run the command to generate CM Deployment template. check out any errors in the console or /tmp/cm_migration/cm_migration.log logs
   Ex: sh am2cm.sh

5. Step6 would generate CMDeploymentTemplate.json file in am2cm folder. 

6. You can validate CM Template and deploy using below API
 http://$CM_HOST:7180/static/apidocs/ui/index.html#!/ClouderaManagerResource/updateDeployment2

7. In Cloudera Manager UI - Once Parcel deployed , activate the parcel.

8. Look out for any configuration errors in services. 

9. Start the services.


## Example of Running the Tool
- Download the repo and build it
```
# git clone https://github.infra.cloudera.com/Starship/am2cm.git
# cd am2cm/
# mvn clean install
```
- We will find “target/am2cm” dir. We can run the tool to convert the blueprint to cm deployment template as following:
```
# cd target/am2cm/
# ./am2cm.sh -bp conf/blueprint.json -dt cm_deployment_template.json -hdp hdp3
```


**NOTE:**
1. Please make sure that the path supplied for blueprint JSON is valid.
2. If you notice the following error while running the tool:
```
Failed to generated CM Template, reason: Blueprint does not have host information
```

Then you will need to check the blueprint JSON file to verify if it has the Hostname added to it or not for various hosts? If not then please add the hostnames manually.
There is an Enhancement Request raised for Ambari DC build to make sure that it can be achieved automatically:
https://hortonworks.jira.com/browse/BUG-123089

AM2CM Tool Exit Codes
----------------------
### Exit code '0'
No errors or exceptions, everything went O.K.
### Exit code '1'
Generic exception happened
### Exit code '2'
Pre-upgrade validation failure
### Exit code '101'
Error downloading blueprint from Ambari
### Exit code '102'
Error saving downloaded blueprint from Ambari to a file
### Exit code '151'
Error reading the deployment template file during upload to Cloudera Manager
### Exit code '152'
Error during deployment template file upload

